use std::sync::Arc;

use rocket::{
    fairing::{Fairing, Info, Kind},
    http::Header,
    Request,
    Response,
};

/// Fairing for adding cache control headers to certain routes
///
/// ## Example
/// ```rust
/// # use cache_control::CacheControl;
/// # use rocket::{
/// #     fs::FileServer,
/// #     Build,
/// #     Rocket,
/// # };
///
/// pub fn init(build: Rocket<Build>) -> Rocket<Build> {
///     build
///         .mount("/img", FileServer::from("./img"))
///         .mount("/js", FileServer::from("./js"))
///         .attach(CacheControl::new(&["/img", "/js"]))
/// }
/// ```
#[derive(Debug)]
pub struct CacheControl {
    routes: Arc<&'static [&'static str]>,
}

impl CacheControl {
    /// Specify the route bases you want to be cached
    pub fn new(routes: &'static [&'static str]) -> Self {
        Self {
            routes: Arc::new(routes),
        }
    }
}

#[rocket::async_trait]
impl Fairing for CacheControl {
    fn info(&self) -> Info {
        Info {
            name: "Cache Control",
            kind: Kind::Response,
        }
    }

    async fn on_response<'r>(&self, req: &'r Request<'_>, res: &mut Response<'r>) {
        if let Some(route) = req.route() {
            if self
                .routes
                .iter()
                .any(|r| *r == route.uri.base.path().as_str())
            {
                let header = Header::new("Cache-Control", "max-age=604800, must-revalidate");
                res.set_header(header);
            }
        }
    }
}
